import json
import random
import sys


def main():
    # Read all stdin
    data = sys.stdin.read()

    # Parse the JSON
    parsed = json.loads(data)

    # Modify some things
    parsed["processed"] = True
    parsed["random"] = random.randint(0, 100)

    # Dump the JSON
    print(json.dumps(parsed))


if __name__ == "__main__":
    main()



import difflib
from dataclasses import dataclass
from enum import Enum


@dataclass
class Difference:
    """
    Represents a difference between expectation and actual.
    """

    expected: any
    """The expected value."""
    actual: any
    """The actual value."""
    location: str = None
    """The location of the difference, if applicable."""
    message: str = None
    """The message of the difference, if applicable."""


class DiffStyle(Enum):
    FULL = "full"
    UNIFIED = "unified"


def _red(text):
    return f"\x1b[38;2;255;0;0m{text}\x1b[38;2;255;255;255m"


def _green(text):
    return f"\x1b[38;2;0;255;0m{text}\x1b[38;2;255;255;255m"


def _blue(text):
    return f"\x1b[38;2;0;0;255m{text}\x1b[38;2;255;255;255m"


def _cyan(text):
    return f"\x1b[38;2;0;255;255m{text}\x1b[38;2;255;255;255m"


def _white(text):
    return f"\x1b[38;2;255;255;255m{text}\x1b[38;2;255;255;255m"


def diff_color_code_full(old: str, new: str) -> str:
    """
    Returns a colored diff between two strings.

    Parameters
    ----------
    old : str
        The old string.
    new : str
        The new string.

    Returns
    -------
    str
        The colored diff.
    """
    result = ""
    codes = difflib.SequenceMatcher(a=old, b=new).get_opcodes()
    for code in codes:
        if code[0] == "equal":
            result += _white(new[code[3] : code[4]])
        elif code[0] == "delete":
            result += _red(old[code[1] : code[2]])
        elif code[0] == "insert":
            result += _green(new[code[3] : code[4]])
        elif code[0] == "replace":
            result += _red(old[code[1] : code[2]]) + _green(new[code[3] : code[4]])
    return result


def diff_color_code_unified(old: str, new: str, n: int = 3, line_sep: str = "\n") -> str:
    """
    Returns a colored diff between two strings based on lines.

    Parameters
    ----------
    old : str
        The old string.
    new : str
        The new string.
    n : int, optional
        The number of lines to show around the change, by default 3.
    line_sep : str, optional
        The line separator, by default "\n".

    Returns
    -------
    str
        The colored diff.
    """
    # Split the strings into lines using the provided line separator
    old_lines = old.split(line_sep)
    new_lines = new.split(line_sep)

    # Use difflib to compare lines instead of characters
    diff = difflib.unified_diff(old_lines, new_lines, n=n, lineterm="")
    result = []

    for line in diff:
        if line.startswith("+"):
            # New line (green)
            result.append(_green(line))
        elif line.startswith("-"):
            # Removed line (red)
            result.append(_red(line))
        elif line.startswith("@@"):
            # Hunk header (cyan or some other highlighting)
            result.append(_cyan(line))
        else:
            # Context line or other lines (white)
            result.append(_white(line))

    # Join all lines with the provided line separator
    return line_sep.join(result)



import glob
import inspect
import json
import os.path
import tempfile
import unittest
from dataclasses import dataclass, field

from goldie.comparison import ConfigComparison, compare, process
from goldie.execution import ConfigRun, ConfigRunValidation, execute
from goldie.update import UPDATE


@dataclass
class TestDefinition:
    """A test definition for a golden file test."""

    input_file: str
    """The input file to use for the test."""
    extra_args: list[tuple[str, str]] = field(default_factory=list)
    """
    Extra arguments to pass to the command.
    These are given as a list of tuples with the placeholder and the value.
    """


@dataclass
class ConfigDirectoryTest:
    """Configuration for directory based golden file testing."""

    comparison_configuration: ConfigComparison
    """The configuration for comparing the actual and golden files."""
    run_configuration: ConfigRun
    """The run configuration to use to run the command."""
    file_filter: str = None
    """The file filter to use to find test files."""
    explicit_files: list[TestDefinition] = field(default_factory=list)
    """A list of explicit files to test."""
    run_validation_configuration: ConfigRunValidation = field(default_factory=lambda: ConfigRunValidation())
    """The run validation configuration to use to validate the command."""


def _get_golden_filename(path: str) -> str:
    """
    Get the golden filename from a path.

    Parameters
    ----------
    path : str
        The path to get the golden filename from.

    Returns
    -------
    str
        The golden filename.
    """
    return path + ".golden"


def _get_caller_directory() -> str:
    """
    Get the directory of the caller's caller.

    Returns
    -------
    str
        The directory.
    """
    abs_path = os.path.abspath((inspect.stack()[2])[1])
    directory_of_1py = os.path.dirname(abs_path)
    return directory_of_1py


def run_unittest(
    test: unittest.TestCase,
    configuration: ConfigDirectoryTest,
):
    """
    Run the golden file test.

    Parameters
    ----------
    test : unittest.TestCase
        The test case to run.
    configuration : ConfigDirectoryTest
        The configuration for the golden file test.
    """

    # Determine the root directory
    root_directory = _get_caller_directory()

    # Find files from file filter
    filter_files = []
    if configuration.file_filter is not None:
        filter_files = glob.glob(os.path.join(root_directory, configuration.file_filter))
        # Remove any golden files
        filter_files = [f for f in filter_files if not f.endswith(".golden")]

    # Convert to test definitions
    test_files = [TestDefinition(input_file) for input_file in filter_files]
    test_files.extend(configuration.explicit_files)

    # Iterate over the test cases
    for i, td in enumerate(test_files):
        with test.subTest(case=i + 1, file=td.input_file), tempfile.NamedTemporaryFile("w+") as output_file:
            # Get the golden file
            golden_file = _get_golden_filename(td.input_file)

            # Run the command
            exit_code = execute(
                input_file=td.input_file,
                output_file=output_file.name,
                cwd=root_directory,
                configuration=configuration.run_configuration,
                extra_args=td.extra_args,
            )

            # Assert the exit code
            if configuration.run_validation_configuration.validate_exit_code:
                test.assertEqual(
                    exit_code,
                    configuration.run_validation_configuration.expected_exit_code,
                    f"Expected exit code {configuration.run_validation_configuration.expected_exit_code}"
                    + f", but got {exit_code}.",
                )

            # Process the file
            process(output_file.name, configuration.comparison_configuration)

            # Update the golden file if necessary
            if UPDATE:
                with open(golden_file, "w") as f:
                    f.write(json.dumps(json.load(output_file), indent=4))
                continue

            # Compare the actual and golden files
            equal, message, differences = compare(output_file.name, golden_file, configuration.comparison_configuration)
            # Prepare the message
            if differences:
                message += "\n" + "\n".join(
                    [f"{d.location}: {d.message} ({d.expected} != {d.actual})" for d in differences]
                )
            # Assert the comparison
            test.assertTrue(equal, message)



from .__about__ import __version__
from .comparison import ComparisonType as ComparisonType
from .comparison import ConfigCompareJson as ConfigCompareJson
from .comparison import ConfigCompareString as ConfigCompareString
from .comparison import ConfigComparison as ConfigComparison
from .comparison import ConfigProcessJson as ConfigProcessJson
from .comparison import ConfigProcessString as ConfigProcessString
from .comparison import JsonReplacement as JsonReplacement
from .comparison import JsonRounding as JsonRounding
from .comparison import RegexReplacement as RegexReplacement
from .comparison import compare as compare
from .diff import Difference as Difference
from .diff import DiffStyle as DiffStyle
from .directory import ConfigDirectoryTest as ConfigDirectoryTest
from .directory import TestDefinition as TestDefinition
from .directory import run_unittest as run_unittest
from .execution import ConfigRun as ConfigRun
from .execution import ConfigRunValidation as ConfigRunValidation
from .execution import InputMode as InputMode
from .execution import OutputMode as OutputMode
from .execution import execute as execute

VERSION = __version__



import subprocess
from dataclasses import dataclass
from enum import Enum


class InputMode(Enum):
    STDIN = "stdin"
    """Feeds the input file via stdin."""
    NONE = "none"
    """Does not feed anything to the stdin."""


class OutputMode(Enum):
    STDOUT = "stdout"
    """Intercepts only stdout."""
    STDERR = "stderr"
    """Intercepts only stderr."""
    BOTH = "both"
    """Intercepts both stdout and stderr."""
    NONE = "none"
    """Does not intercept anything, i.e., ignores the output."""


@dataclass
class ConfigRun:
    """Configuration for running a command."""

    cmd: str
    """The command to run."""
    args: list[str]
    """
    The arguments to pass to the command.
    If the path to the input file is needed (instead of feeding via stdin), use the string "{input}" as a placeholder.
    If the path to the output file is needed (instead of reading stdout), use the string "{output}" as a placeholder.
    If there are further placeholders, use the string "{name}" as a placeholder and provide the values in the test
    definition.
    """
    cwd: str = None
    """
    The directory to run the command in.
    If none, the directory of the test caller is used to match golden file filters.
    """
    input_mode: InputMode = InputMode.STDIN
    """The input mode."""
    output_mode: OutputMode = OutputMode.STDOUT
    """The output mode."""


@dataclass
class ConfigRunValidation:
    """Configuration for validating the command"""

    validate_exit_code: bool = True
    """Whether to validate the exit code of the command."""
    expected_exit_code: int = 0
    """The desired exit code of the command."""


def execute(
    input_file: str,
    output_file: str,
    cwd: str,
    configuration: ConfigRun,
    extra_args: list[tuple[str, str]] = None,
) -> int:
    """
    Run the command with the input file and return the result.

    Parameters
    ----------
    input_file : str
        The file to read the input from.
    output_file : str
        The file to write the output to.
    cwd : str
        The directory to run the command in.
    configuration : ConfigRun
        The configuration for running the command.
    extra_args : list[tuple[str, str]], optional
        Extra arguments to pass to the command. Each tuple should contain the placeholder
        (needs to match the one in args of configuration) and the value.

    Returns
    -------
    int
        The exit code of the command.
    """
    # Initialize the extra arguments if necessary
    extra_args = extra_args or []
    # Replace the placeholders in the arguments
    args = [arg.format(input=input_file, output=output_file, **dict(extra_args)) for arg in configuration.args]

    # Run the command
    with open(output_file, "w") as f:
        input_file = None if configuration.input_mode == InputMode.NONE else open(input_file)
        process = subprocess.run(
            [configuration.cmd, *args],
            stdin=input_file if configuration.input_mode == InputMode.STDIN else None,
            stdout=f if configuration.output_mode in [OutputMode.STDOUT, OutputMode.BOTH] else None,
            stderr=f if configuration.output_mode in [OutputMode.STDERR, OutputMode.BOTH] else None,
            cwd=cwd if configuration.cwd is None else configuration.cwd,
        )

    # Close the input file if necessary
    if input_file is not None:
        input_file.close()

    # Return the exit code and the path to the output file
    return process.returncode
